import { name, autoService } from 'knifecycle';
import { YError } from 'yerror';
import fs from 'fs';
import { mkdirp } from 'mkdirp';
async function initFS({ programOptions, log, }) {
    return {
        mkdirpAsync: async (path) => {
            if (programOptions.dryRun) {
                log('warning', '📂 - Create a folder:', path);
                return;
            }
            await mkdirp(path, {
                fs: {
                    mkdir: ((...args) => {
                        if (programOptions.safe) {
                            throw new YError('E_UNEXPECTED_CHANGES', args[0]);
                        }
                        fs.mkdir(...args);
                    }),
                    stat: fs.stat,
                },
            });
        },
        readFileAsync: fs.promises.readFile,
        accessAsync: fs.promises.access,
        readdirAsync: fs.promises.readdir,
        unlinkAsync: async (path) => {
            if (programOptions.dryRun) {
                log('warning', '⌫ - Delete a file:', path);
                return;
            }
            if (programOptions.safe) {
                throw new YError('E_UNEXPECTED_CHANGES', path);
            }
            await fs.promises.unlink(path);
        },
        writeFileAsync: async (path, data) => {
            if (programOptions.dryRun) {
                log('warning', '💾 - Modify a file:', path);
                return;
            }
            if (programOptions.safe) {
                throw new YError('E_UNEXPECTED_CHANGES', path);
            }
            await fs.promises.writeFile(path, data);
        },
        constants: fs.constants,
    };
}
export default name('fs', autoService(initFS));
//# sourceMappingURL=fs.js.map