import path from 'path';
import { autoService } from 'knifecycle';
import { mapConfigsSequentially, identity } from '../libs/utils.js';
import { printStackTrace } from 'yerror';
export default autoService(initBuildPackageGitHooks);
async function initBuildPackageGitHooks({ ENV, PROJECT_DIR, GIT_HOOKS_DIR, fs, EOL, log, importer, }) {
    return async (packageConf, metapakContext) => {
        // Avoiding CI since it does not make sense
        if (ENV.CI) {
            return;
        }
        // Avoid adding hooks for package that ain't at the git
        // root directory
        if (path.relative(PROJECT_DIR, GIT_HOOKS_DIR).startsWith('..')) {
            return;
        }
        const hooksBuilders = await mapConfigsSequentially(metapakContext, async (metapakModuleName, metapakConfigName) => {
            const packageHooksPath = path.join(metapakContext.modulesConfigs[metapakModuleName].base, metapakContext.modulesConfigs[metapakModuleName].srcDir, metapakConfigName, 'hooks.js');
            try {
                return (await importer(packageHooksPath)).default;
            }
            catch (err) {
                log('debug', '🤷 - No hooks found at:', packageHooksPath);
                log('debug-stack', printStackTrace(err));
            }
            return identity;
        });
        const hooks = await hooksBuilders.reduce((hooks, hooksBuilder) => hooksBuilder(hooks, packageConf), {});
        await Promise.all(Object.keys(hooks).map(async (hookName) => {
            const hookContent = '#!/bin/sh' +
                EOL +
                '# Automagically generated by metapak, do not change in place.' +
                EOL +
                '# Your changes would be loose on the next npm install run.' +
                EOL +
                hooks[hookName].join(';' + EOL);
            const hookPath = path.join(GIT_HOOKS_DIR, hookName);
            let currentHookContent = '';
            try {
                currentHookContent = (await fs.readFileAsync(hookPath)).toString();
            }
            catch (err) {
                log('debug', '🤷 - No existing hook found:', hookPath);
                log('debug-stack', printStackTrace(err));
            }
            if (currentHookContent !== hookContent) {
                await fs.writeFileAsync(hookPath, Buffer.from(hookContent), {
                    mode: 0o777,
                });
            }
        }));
    };
}
//# sourceMappingURL=gitHooks.js.map